#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 21 10:16:58 2025

@author: pteysseyre
"""

''' Manages the file where the breakpoints and quiet amplitude and phase are written.'''

import os.path
import numpy as np
import pandas as pd

def get_lastbreakpoint(la_station, la_date, path, quiet=False):
    
    ''' Read the value of the last breakpoint, such as the time of detection, the slope, and the associated amplitude and phase.
    If quiet is True, it returns the values for the last quiet breakpoint.
    
    If there were no breakpoints today, return a series of zeros and create the file'''
    
    # Read the last breakpoint value OR create a new file if it is the first of the day
    # If there are no lines yet in the file, write header
    if not os.path.isfile(path+la_date+'_'+la_station+'.csv'):
        
        data = [[0,0,0,0,1, 0]]
        df = pd.DataFrame(data, columns = ['Time (UT)', 'Slope (deg/hr)', 'Amplitude (pT)', 'Phase (deg)', 'Quiet', 'DP'])
        df.to_csv(path+la_date+'_'+la_station+'.csv', index=False)
        
    else :
        df = pd.read_csv(path+la_date+'_'+la_station+'.csv')
    
    if quiet:
        df = df.loc[df['Quiet']==1]
        
    last_bp = df.iloc[-1]
        
    return last_bp['Time (UT)'], last_bp['Slope (deg/hr)'], last_bp['Amplitude (pT)'], last_bp['Phase (deg)'], last_bp['Quiet'], last_bp['DP']

def write_newbreakpoint(la_station, la_date, path, time_now, p1, amp, phase, quiet, DP):
    
    ''' Update the state file by adding the new detected breakpoint.
    Note: because the function get_lastbreakpoint has been called earlier in the Realtimescript, there
    is no need to check that a file exist, or to create a new one'''
    
    # Read the state file 
    df = pd.read_csv(path+la_date+'_'+la_station+'.csv')
    
    # Add the new values 
    df.loc[len(df)] = [time_now, p1, amp, phase, quiet, DP]
    
    # Write the new file
    df.to_csv(path+la_date+'_'+la_station+'.csv', index=False)
    
    return


import numpy as np
import vlf4ions.compute_electron_density as ced
import vlf4ions.class_definition as cd
import os
dirname = os.path.dirname(__file__)

def test_get_quiet_hb():
    sza_1 = 0 
    H_1, B_1 = ced.get_quiet_hb(sza_1)

    sza_2 = np.pi/4
    H_2, B_2 = ced.get_quiet_hb(sza_2)

    sza_3 = 45
    H_3, B_3 = ced.get_quiet_hb(sza_3)

    assert((np.abs(H_1 - 70.55) < 1e-4) and (np.abs(B_2 - 0.3668) < 1e-4) and (H_2 == H_3))

def test_LMP_findminimum():
    sza = 66.7753
    GQD = cd.station('GQD', 54.73, -2.88, 22.1, 24.4, ':45')
    A_quiet = 10.7639414407342
    P_quiet = 110.027930269667
    DA = 10.556409437684
    DP = 143.680907339525

    path_to_files = os.path.join(dirname, '')

    H, B = ced.LMP_findminimum(GQD, DA, DP, A_quiet, P_quiet, sza, path_to_files)
    print(H, B)

    assert((np.abs(H - 79.3) < 0.1) and (np.abs(B - 0.21) < 0.001))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 15 14:48:32 2025

@author: pteysseyre
"""

import pandas as pd
import matplotlib.pyplot as plt
import matplotlib as mpl
from mpl_toolkits.basemap import Basemap
import matplotlib.cm as cm

def plot_map(stations_l, thisreceiver, today, Nemax=1e10):

    '''Plots the current state on a geographical map'''

    fig, ax = plt.subplots(1, 1, figsize=(6, 8))  # setup the plot
    m = Basemap(llcrnrlon=-72,
            llcrnrlat=35,
            urcrnrlon=25,
            urcrnrlat=60,
            projection='merc')
    m.drawmapboundary(fill_color='white', linewidth=0)
    m.fillcontinents(color='grey', alpha=0.7, lake_color='grey')
    m.drawcoastlines(linewidth=0.1, color="white")
    
    norm = mpl.colors.LogNorm(vmin=1e8, vmax=Nemax)
    cmap = cm.RdYlGn_r
    sm = cm.ScalarMappable(norm=norm, cmap=cmap)
    m.nightshade(today, alpha=0.25)
        

    for st in stations_l:

        color  = sm.to_rgba(st.Ne)

        m.drawgreatcircle(st.lon, st.lat, thisreceiver.lon, thisreceiver.lat, linestyle = '-',  linewidth=3, color=color)
        
        if st.name == 'GQD':
            plt.annotate(st.name, xy=m(st.lon+3, st.lat), xytext =(-40, -4), textcoords='offset points', fontsize = 12)
        else:
            plt.annotate(st.name, xy=m(st.lon+3, st.lat), xytext =(1, -5), textcoords='offset points', fontsize = 12)

    
    
    sm = cm.ScalarMappable(norm=norm, cmap=cmap)
    sm.set_array([]) # can be an empty list, only needed for matplotlib < 3.1
    # ...
    cb = fig.colorbar(sm, ax=ax, orientation = 'horizontal', pad = 0.02)
    cb.set_label('Ne ($m^{-3}$)', rotation=360)

    return m


def get_latlong(la_station, file):
    ''' Returns the latitude and longitude of the station considered, from the 'file' given as input'''

    stations = pd.read_csv(file)

    station = stations[stations.Callsign == la_station]
    lat = station.iloc[0]['Latitude']
    long = station.iloc[0]['Longitude']
    Ne = station.iloc[0]['Ne']

    return lat, long, Ne





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 10:00:48 2024

@author: pteysseyre
"""

from setuptools import find_packages, setup

setup(
    name='vlf4ions',
    packages=find_packages(include=['vlf4ions']),
    version='1.2.1',
    description='Functions necessary to analyse in real time the output of an AWESOME antenna and detect any solar flare',
    author='P. Teysseyre',
    install_requires=['pymatreader', 
                      'datetime', 
                      'schedule', 
                      'pre-commit', 
                      'pandas', 
                      'numpy==1.26.4', 
                      'basemap', 
                      'pysolar',
                      'pytz',
                      ],
)


import vlf4ions.class_definition as cd
import numpy as np

def test_email_alert_update_body():
    test_alert = cd.email_alerts('test_subject', 'test_body', 'test_sender', 'test_password', ['test_recipients'])
    test_alert.update_detection_body('NRK', 0)
    #print(test_alert.body)
    assert((test_alert.body[0:18] == 'Detection with NRK') and (test_alert.body[-9:] == '0 deg/hr.'))


# TODO: test send_email

def test_sliding_median():
    test_array = np.arange(6)
    res = cd.sliding_median(test_array, 3)

    assert(all(res == [1., 2., 3., 4.]))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  7 14:28:49 2025

@author: pteysseyre
"""

import numpy as np


def analyse_breakpoint(current_time, current_p1, la_station, amp, phase,
                       the_time, quiet, quiet_phase, quiet_amp, last_time,
                       last_p1, detection_threshold, DP):

    """This is the part of the code that will decide if the  breakpoint is a
    flare signature or not.

    Several cases can occur:

        1) This is the beginning of a flare. In this case, we can estimate the
        current X-ray strength and electron density by taking as a reference
        the last 'quiet' breakpoint. We can also estimate the state of the
        ionosphere in 1 and 5 minassuming that the trend doesn't change

        2) This is the first brekapoint to signal a flare decay. We can
        estimate the current X-ray flux, electron density, and give an estimate
        of when things will have decreased to half of their increase

        3) This is a quiet point: this is either a slight positive slope, a
        negative slope after a quiet breakpoint, or a negative
        slope after a flare has decayed by more than half of its increase

        4) This is a flare decay, but this is not the first detection and the
        flare has not gone down by more than half of its increase. We can
        update the estimated X-ray flux and Ne, and estimate the new time to
        the end of the flare

        Parameters:
        ---------------------------

        - la_station, phase: the transmitter, recorded & corrected phase (in °)
          since beginning of file
        - amp: median over the last minute of amplitude data.
        - detection_threshold: Standard deviation of slopes in quiet times
        - the_time: time-array. Dimension should match  phase
        - pathtoCSVfiles: path to the CSV files obtained with the LMP model
        - current_p1, current_time : current breakpoint slope detected and
            time  of detection
        - last_p1, last_time: last  breakpoint slope and detection time
        - quiet: is True if the last brealpoint detected was a quiet
            breakpoint. Is False otherwise (flares)
        - quiet_phase, quiet_amp: last read phase & amp (to avoid reading it
            and looking for it)

        Outputs:
        ---------------------------

        - quiet_now : is True if the current breakpoint is in quiet time
        - flux : estimated X-ray flux and confidence interval
        - prev_1min : estimated Ne in 1 min
        - prev_5min : estimated Ne in 5 min
        - prev_timedecay : estimated time of decay


        IMPORTANT NOTE:
        --------------------------
        1) Right now (Feb. 2025), we are concentrating on the phase (Ne in code
        is phase). However, we can for a second step concentrate on the
        electron density at an altitude of 70 km; moreover, the interesting
        quantity for us is the HF absorption. Thus, in the future, the HF
        absorption should be computed and the time of the flare's decay, as
        well as the previsions for the next minutes or so should be based
        on this quantity instead."""

    # TODO: Think about the flares happening in a very fast sucession
    # Init: most of what is returned will not be systematically useful
    prev_timedecay, prev_1min, prev_5min = np.zeros(3)
    quiet_now = 0

    DP_now = phase[-1] - quiet_phase

    if current_p1 > 0:

        if (current_p1 < detection_threshold):

            if (quiet == 1):  # This is a quiet time
                quiet_now = 1
                DP = 0

            else:  # That's either a flare max OR a flare that has decayed

                # We look for the phase maximum to get DP
                the_time, phase = _keep_between(phase, the_time, last_time,
                                                current_time)
                ind_max = np.argmax(phase)
                DP = phase[ind_max] - quiet_phase
                time_max = the_time[ind_max]

                if DP_now < DP/2:  # The flare has already decayed
                    quiet_now = 1
                    DP = 0
                else:  # This is the max of the flare
                    quiet_now = 0
                    prev_1min = DP
                    prev_5min = DP

        else:  # This is a flare

            # TODO: change for Ne or abs
            DA = amp - quiet_amp

            # Compute Ne here
            # TODO: Estimate X-ray flux here

            # Estimate phase in 1 or 5 min
            prev_1min = current_p1*1/60 + phase[-1]
            prev_5min = current_p1*5/60 + phase[-1]


    else:  # This is either quiet time, or a flare's decay

        if quiet:  # Last breakpoint is quiet
            quiet_now = True

        else:  # Last breakpoint was a flare decay

            if last_p1 > 0:  # Last breakpoint was a flare onset

                ### TODO: Use Ne instead of phase

                # We look for the phase maximum to get DP
                the_time, phase = _keep_between(phase, the_time, last_time, 
                                                current_time)
                ind_max = np.argmax(phase)
                DP = phase[ind_max] - quiet_phase
                time_max = the_time[ind_max]

                # We estimate the decay time assuming that the trend is 
                # constant & the decay linear
                prev_timedecay = -DP/(2*current_p1) + time_max

            else:

                # Check if the flare has decayed 
                # (lost more than half its increase)
                ### TODO: change for Ne or abs

                if DP_now < DP/2:  # The flare has ended
                    quiet_now = True

                else:  # The flare is ongoing but we can update our previsions
                    prev_timedecay = 1/current_p1*(DP/2 - DP_now) 
                    + the_time[-1]



    return (quiet_now, prev_1min, prev_5min, prev_timedecay, DP)



def _keep_between(data, the_time, time_inf, time_sup):

    """ This is just a short bit of code to constrain the data between 
    time_inf & time_sup """

    if len(data) != len(the_time):

        print('Careful, lengths do not match - keep_between')

    else:

        ind_inf = np.argmin(np.abs(the_time - time_inf))
        ind_sup = np.argmin(np.abs(the_time - time_sup))

        the_time = the_time[ind_inf:ind_sup]
        data = data[ind_inf:ind_sup]

        return the_time, data















#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 10:29:16 2024

@author: pteysseyre
"""

import numpy as np
from datetime import datetime # To get today's date
import time # Only needed here so that the script keeps running even when it's not computing anything
import os.path # Useful to determine if a file exist or not (to write headers in a new file)
import glob

def correctionphase(station, phase, the_time):
    """Completely corrects the phase in real time the phase so that:
    # 1) All phase jumps from -180° to 180° are corrected, and the phase detrended
    # 2) The absolute value of the phase is also given 

    # CURRENT hypothesis: (08/24)
    # 1) The value of the phase in quiet time during the day is between 90° and 180° 

    # CURRENT STATE : 
    # The phase is currently only unwrapped and detrended
    """
    
    phase_corr, jumps = correctionphaseparDw(station, phase, the_time)
    
    return phase_corr


def unwrap_moco(Data_In, UnwrapNumber):
    """Originally written by M. Cohen in Matlab. Unwraps 'Data_in', 
    by compensating any jumps more than UnwrapNumer/2"""
    
    # Remove all NaN point and reconcatanate
    Data_NaN = np.isnan(Data_In)
    Data_In_NonNaN = Data_In[Data_NaN == False]
    Indices_NonNaN = np.arange(0, np.size(Data_In_NonNaN))
    
    # Take differential
    Data_In_NonNaN_Diff = np.diff(Data_In_NonNaN)
    
    # Identify points where signal jumps by more than unwrap/2 threshold
    Indices_NonNaN = Indices_NonNaN[0:-1]
    JumpIndices = Indices_NonNaN[np.abs(Data_In_NonNaN_Diff)>UnwrapNumber/2]
    Data_In_NonNaN_Diff_Jumps = Data_In_NonNaN_Diff[JumpIndices]
    
    # Round jumps to nearest value of unwrap number
    Data_In_NonNaN_Diff_Jumps = np.round(Data_In_NonNaN_Diff_Jumps/UnwrapNumber) * UnwrapNumber
    
    # Integrates jump to original concatanated vector. Add 1 to indices to reference later point of two differential elements
    Indices_NonNaN_Jump = Indices_NonNaN[JumpIndices]
    
    # Go back to original version with the NaNs and create derivative
    Data_In_Diff_Jumps = Data_In*0
    Data_In_Diff_Jumps[np.isnan(Data_In_Diff_Jumps)] = 0
    Data_In_Diff_Jumps[Indices_NonNaN_Jump] = Data_In_NonNaN_Diff_Jumps
    
    # Integrates to get the function that needs to be substracted
    Data_In_Jumps = np.cumsum(Data_In_Diff_Jumps)
    
    # Substract from original 
    Data_Out = Data_In - Data_In_Jumps
    
    # nb_jumps_station = np.size(JumpIndices)
    
    return Data_Out, Data_In_Jumps[-1]

def correctionphaseparDw(station, phase, the_time):
    """Corrects the phase by compensating for the phase detrend induced by the 
    transmitter frequency not being exactly stable
    
    NOTE : The values were obtained from the function 'find_dw' written in Matlab
    They are only valid for the stations considered below, and only for the Nançay 
    AWESOME antenna"""

    Deltaf = station.df
        
    Deltaw = 2*np.pi*Deltaf
    
    phase_un, jumps = unwrap_moco(phase, 90) 
    
    phase_corr = phase_un + (Deltaw*the_time*3600)*180/np.pi
    
    # For now (03/24) because there is both a 90° uncertainty on the measurements and the modelling
    # we choose to keep the phase between 90 and 180° for quiet daytime conditions
    
    
    return phase_corr, jumps

